function BitWriter () {
	var self = this;
	var output = "";
	var nbBit = 0;
	var nb = 0;
	
	self.putBit = function (bit) {
		nb = (nb << 1) + ((bit) ? 1 : 0);
		nbBit++;
		
		if (nbBit === 8) {
			output += String.fromCharCode(nb);
			nb = 0;
			nbBit = 0;
		}
	};
	
	self.toString = function () {
		var extraPad = 0;
		
		while (nbBit !== 0) {
			extraPad++;
			self.putBit(false);
		}
		
		return extraPad + output;
	};
}

function BitReader (str) {
	var self = this;
	
	var position = 1;
	var bitPosition = -1;
	var currentInt;
	var extraPad = +str[0];
	
	self.getBit = function () {
		var result;
		
		if (bitPosition === -1) {
			currentInt = str.charCodeAt(position++);
			bitPosition = 7;
		}
		
		result = !!((currentInt >> bitPosition) & 1);
		bitPosition--;
		
		return result;
	};
	
	self.hasLeft = function () {
		if (position < str.length - 1) {
			return true;
		}
		
		if (position === str.length - 1) {
			return bitPosition >= (extraPad -1);
		}
		
		return false;
	};
}

function GolombCompression () {
	var self = this;
	
	self.decode = function (source, M) {
		var bitreader = new BitReader(source);
		var strResult = "";
		
		var q = 0;
		var nr = 0;
		var end = Math.log(M) / Math.log(2);
		
		while (bitreader.hasLeft())
		{
			nr = 0;
			q = 0;
			
			while (bitreader.getBit()) {
				q++;
			}
			
			for (var a = 0; a < end; a++) {   // read out the sequential log2(M) bits
				if (bitreader.getBit()) {
					nr += 1 << a;
				}
			}
			
			nr += q * M; // add the bits and the multiple of M
			strResult += String.fromCharCode(nr);
		}
		
		return strResult;
	};
	
	self.encode = function (source, M) {
		var bitWriter = new BitWriter();
		var end = Math.log(M) / Math.log(2);
		
		for (var i=0; i<source.length; i++) {
			var num = source.charCodeAt(i);
			var q = Math.floor(num / M);
			
			for (var j = 0 ; j < q; j++) {
				bitWriter.putBit(true);   // write q ones
			}
			
			bitWriter.putBit(false);      // write one zero
			var v = 1;
			
			for (var j = 0 ; j < end; j++) {            
				bitWriter.putBit(!!(v & num));
				v = v << 1;
			}
		}
		
		return bitWriter.toString();
	};
}